/*	Renegade Scripts.dll
	Base defence scripts
	Copyright 2006 Olaf Van Der Spek, Jonathan Wilson

	This file is part of the Renegade scripts.dll
	The Renegade scripts.dll is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
	In addition, an exemption is given to allow Run Time Dynamic Linking of this code with any closed source module that does not contain code covered by this licence.
	Only the source code to the module(s) containing the licenced code has to be released.
*/
#include "scripts.h"
#include "engine.h"
#include "jfwdef.h"

void JFW_User_Controllable_Base_Defence::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	objtype = Commands->Get_Player_Type(obj);
	player = false;
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_User_Controllable_Base_Defence::Custom(GameObject *obj,int message,int param,GameObject *sender)
{
	ActionParamsStruct params;
	if (message == CUSTOM_EVENT_VEHICLE_EXIT)
	{
		player = false;
		Commands->Set_Player_Type(obj,objtype);
		Commands->Action_Reset(obj,100);
	}
	else
	{
		if (message == CUSTOM_EVENT_VEHICLE_ENTER)
		{
			player = true;
			params.Set_Basic(this,100,3,-1);
			Commands->Action_Follow_Input(obj,params);
		}
	}
}

void JFW_User_Controllable_Base_Defence::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	ActionParamsStruct params;
	Vector3 pos,pos2;
	float distance;
	float minattack,maxattack;
	float attacktimer;
	if (!player)
	{
		pos = Commands->Get_Position(obj);
		pos2 = Commands->Get_Position(seen);
		distance = Commands->Get_Distance(pos2,pos);
		minattack = Get_Float_Parameter("MinAttackDistance");
		if (distance >= minattack)
		{
			maxattack = Get_Float_Parameter("MaxAttackDistance");
			params.Set_Basic(this,100,2,-1);
			params.Set_Attack_Hold(seen,maxattack,0.0,true,false);
			Commands->Action_Attack(obj,params);
			attacktimer = Get_Float_Parameter("AttackTimer");
			Commands->Start_Timer(obj,this,attacktimer,2);
		}
	}
}

void JFW_User_Controllable_Base_Defence::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_User_Controllable_Base_Defence::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		if (!player)
		{
			switch (__min(int(f),2))
			{
			case 0:
				object = Commands->Find_Object(id1);
				if (object)
				{
					var.Set_Basic(this,70,1,-1);
					var.Set_Attack(object,0.0,0.0,true);
					Commands->Action_Attack(obj,var);
				}
				break;
			case 1:
				object = Commands->Find_Object(id2);
				if (object)
				{
					var.Set_Basic(this,70,1,-1);
					var.Set_Attack(object,0.0,0.0,true);
					Commands->Action_Attack(obj,var);
				}
				break;
			default:
				object = Commands->Find_Object(id3);
				if (object)
				{
					var.Set_Basic(this,70,1,-1);
					var.Set_Attack(object,0.0,0.0,true);
					Commands->Action_Attack(obj,var);
				}
			}
		}
		Commands->Start_Timer(obj,this,10,1);
		break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_User_Controllable_Base_Defence::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
	Auto_Save_Variable(5,1,&player);
}

void JFW_Base_Defence::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	objtype = Commands->Get_Player_Type(obj);
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_Base_Defence::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	ActionParamsStruct params;
	Vector3 pos,pos2;
	float distance;
	float minattack,maxattack;
	float attacktimer;
	pos = Commands->Get_Position(obj);
	pos2 = Commands->Get_Position(seen);
	distance = Commands->Get_Distance(pos2,pos);
	minattack = Get_Float_Parameter("MinAttackDistance");
	if (distance >= minattack)
	{
		maxattack = Get_Float_Parameter("MaxAttackDistance");
		params.Set_Basic(this,100,2,-1);
		params.Set_Attack_Hold(seen,maxattack,0.0,true,false);
		Commands->Action_Attack(obj,params);
		attacktimer = Get_Float_Parameter("AttackTimer");
		Commands->Start_Timer(obj,this,attacktimer,2);
	}
}

void JFW_Base_Defence::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_Base_Defence::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		switch (__min(int(f),2))
		{
		case 0:
			object = Commands->Find_Object(id1);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		case 1:
			object = Commands->Find_Object(id2);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		default:
			object = Commands->Find_Object(id3);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
		}
	Commands->Start_Timer(obj,this,10,1);
	break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_Base_Defence::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
}

void JFW_Base_Defence_No_Aircraft::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	objtype = Commands->Get_Player_Type(obj);
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_Base_Defence_No_Aircraft::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		ActionParamsStruct params;
		Vector3 pos,pos2;
		float distance;
		float minattack,maxattack;
		float attacktimer;
		pos = Commands->Get_Position(obj);
		pos2 = Commands->Get_Position(seen);
		distance = Commands->Get_Distance(pos2,pos);
		minattack = Get_Float_Parameter("MinAttackDistance");
		if (distance >= minattack)
		{
			maxattack = Get_Float_Parameter("MaxAttackDistance");
			params.Set_Basic(this,100,2,-1);
			params.Set_Attack_Hold(seen,maxattack,0.0,true,false);
			Commands->Action_Attack(obj,params);
			attacktimer = Get_Float_Parameter("AttackTimer");
			Commands->Start_Timer(obj,this,attacktimer,2);
		}
	}
}

void JFW_Base_Defence_No_Aircraft::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_Base_Defence_No_Aircraft::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		switch (__min(int(f),2))
		{
		case 0:
			object = Commands->Find_Object(id1);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		case 1:
			object = Commands->Find_Object(id2);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		default:
			object = Commands->Find_Object(id3);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
		}
	Commands->Start_Timer(obj,this,10,1);
	break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_Base_Defence_No_Aircraft::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
}

void JFW_Base_Defence_Aircraft_Only::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	objtype = Commands->Get_Player_Type(obj);
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_Base_Defence_Aircraft_Only::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		ActionParamsStruct params;
		Vector3 pos,pos2;
		float distance;
		float minattack,maxattack;
		float attacktimer;
		pos = Commands->Get_Position(obj);
		pos2 = Commands->Get_Position(seen);
		distance = Commands->Get_Distance(pos2,pos);
		minattack = Get_Float_Parameter("MinAttackDistance");
		if (distance >= minattack)
		{
			maxattack = Get_Float_Parameter("MaxAttackDistance");
			params.Set_Basic(this,100,2,-1);
			params.Set_Attack_Hold(seen,maxattack,0.0,true,false);
			Commands->Action_Attack(obj,params);
			attacktimer = Get_Float_Parameter("AttackTimer");
			Commands->Start_Timer(obj,this,attacktimer,2);
		}
	}
}

void JFW_Base_Defence_Aircraft_Only::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_Base_Defence_Aircraft_Only::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		switch (__min(int(f),2))
		{
		case 0:
			object = Commands->Find_Object(id1);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		case 1:
			object = Commands->Find_Object(id2);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		default:
			object = Commands->Find_Object(id3);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
		}
	Commands->Start_Timer(obj,this,10,1);
	break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_Base_Defence_Aircraft_Only::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
}

void JFW_Base_Defence_Secondary::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	primary = true;
	objtype = Commands->Get_Player_Type(obj);
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_Base_Defence_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	ActionParamsStruct params;
	Vector3 pos,pos2;
	float distance;
	float minattack,maxattack;
	float attacktimer;
	pos = Commands->Get_Position(obj);
	pos2 = Commands->Get_Position(seen);
	distance = Commands->Get_Distance(pos2,pos);
	minattack = Get_Float_Parameter("MinAttackDistance");
	if (distance >= minattack)
	{
		maxattack = Get_Float_Parameter("MaxAttackDistance");
		params.Set_Basic(this,100,2,-1);
		params.Set_Attack_Hold(seen,maxattack,0.0,primary,false);
		Commands->Action_Attack(obj,params);
		attacktimer = Get_Float_Parameter("AttackTimer");
		Commands->Start_Timer(obj,this,attacktimer,2);
		if (primary)
		{
			primary = false;
		}
		else
		{
			primary = true;
		}
	}
}

void JFW_Base_Defence_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_Base_Defence_Secondary::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		switch (__min(int(f),2))
		{
		case 0:
			object = Commands->Find_Object(id1);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		case 1:
			object = Commands->Find_Object(id2);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		default:
			object = Commands->Find_Object(id3);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
		}
	Commands->Start_Timer(obj,this,10,1);
	break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_Base_Defence_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
	Auto_Save_Variable(5,1,&primary);
}

void JFW_Base_Defence_No_Aircraft_Secondary::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	primary = true;
	objtype = Commands->Get_Player_Type(obj);
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_Base_Defence_No_Aircraft_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		ActionParamsStruct params;
		Vector3 pos,pos2;
		float distance;
		float minattack,maxattack;
		float attacktimer;
		pos = Commands->Get_Position(obj);
		pos2 = Commands->Get_Position(seen);
		distance = Commands->Get_Distance(pos2,pos);
		minattack = Get_Float_Parameter("MinAttackDistance");
		if (distance >= minattack)
		{
			maxattack = Get_Float_Parameter("MaxAttackDistance");
			params.Set_Basic(this,100,2,-1);
			params.Set_Attack_Hold(seen,maxattack,0.0,primary,false);
			Commands->Action_Attack(obj,params);
			attacktimer = Get_Float_Parameter("AttackTimer");
			Commands->Start_Timer(obj,this,attacktimer,2);
			if (primary)
			{
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_No_Aircraft_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_Base_Defence_No_Aircraft_Secondary::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		switch (__min(int(f),2))
		{
		case 0:
			object = Commands->Find_Object(id1);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		case 1:
			object = Commands->Find_Object(id2);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		default:
			object = Commands->Find_Object(id3);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
		}
	Commands->Start_Timer(obj,this,10,1);
	break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_Base_Defence_No_Aircraft_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
	Auto_Save_Variable(5,1,&primary);
}

void JFW_Base_Defence_Aircraft_Only_Secondary::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	primary = true;
	objtype = Commands->Get_Player_Type(obj);
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_Base_Defence_Aircraft_Only_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		ActionParamsStruct params;
		Vector3 pos,pos2;
		float distance;
		float minattack,maxattack;
		float attacktimer;
		pos = Commands->Get_Position(obj);
		pos2 = Commands->Get_Position(seen);
		distance = Commands->Get_Distance(pos2,pos);
		minattack = Get_Float_Parameter("MinAttackDistance");
		if (distance >= minattack)
		{
			maxattack = Get_Float_Parameter("MaxAttackDistance");
			params.Set_Basic(this,100,2,-1);
			params.Set_Attack_Hold(seen,maxattack,0.0,primary,false);
			Commands->Action_Attack(obj,params);
			attacktimer = Get_Float_Parameter("AttackTimer");
			Commands->Start_Timer(obj,this,attacktimer,2);
			if (primary)
			{
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_Aircraft_Only_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_Base_Defence_Aircraft_Only_Secondary::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		switch (__min(int(f),2))
		{
		case 0:
			object = Commands->Find_Object(id1);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		case 1:
			object = Commands->Find_Object(id2);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		default:
			object = Commands->Find_Object(id3);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
		}
	Commands->Start_Timer(obj,this,10,1);
	break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_Base_Defence_Aircraft_Only_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
	Auto_Save_Variable(5,1,&primary);
}

void JFW_Base_Defence_Animated::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (!popup)
	{
		popup = true;
		Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
		Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
	}
	else if (attack)
	{
		ActionParamsStruct params;
		params.Set_Basic(this,100,1,-1);
		params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,true,false);
		Commands->Action_Attack(obj,params);
	}
}

void JFW_Base_Defence_Animated::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&attack);
}

void JFW_Base_Defence_Animated_No_Aircraft::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_No_Aircraft::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,true,false);
			Commands->Action_Attack(obj,params);
		}
	}
}

void JFW_Base_Defence_Animated_No_Aircraft::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_No_Aircraft::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_No_Aircraft::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&attack);
}

void JFW_Base_Defence_Animated_Aircraft_Only::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Aircraft_Only::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,true,false);
			Commands->Action_Attack(obj,params);
		}
	}
}

void JFW_Base_Defence_Animated_Aircraft_Only::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Aircraft_Only::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Aircraft_Only::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&attack);
}

void JFW_Base_Defence_Animated_Secondary::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	primary = true;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (!popup)
	{
		popup = true;
		Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
		Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
	}
	else if (attack)
	{
		ActionParamsStruct params;
		params.Set_Basic(this,100,1,-1);
		params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,primary,false);
		Commands->Action_Attack(obj,params);
		if (primary)
		{
			primary = false;
		}
		else
		{
			primary = true;
		}
	}
}

void JFW_Base_Defence_Animated_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Secondary::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&primary);
	Auto_Save_Variable(3,1,&attack);
}

void JFW_Base_Defence_Animated_No_Aircraft_Secondary::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	primary = true;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_No_Aircraft_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,primary,false);
			Commands->Action_Attack(obj,params);
			if (primary)
			{
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_Animated_No_Aircraft_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_No_Aircraft_Secondary::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_No_Aircraft_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&primary);
	Auto_Save_Variable(3,1,&attack);
}

void JFW_Base_Defence_Animated_Aircraft_Only_Secondary::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	primary = true;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Aircraft_Only_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,primary,false);
			Commands->Action_Attack(obj,params);
			if (primary)
			{	
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_Animated_Aircraft_Only_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Aircraft_Only_Secondary::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Aircraft_Only_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&primary);
	Auto_Save_Variable(3,1,&attack);
}

void JFW_Base_Defence_No_VTOL::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	objtype = Commands->Get_Player_Type(obj);
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_Base_Defence_No_VTOL::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (!Is_VTOLVehicle(seen))
	{
		ActionParamsStruct params;
		Vector3 pos,pos2;
		float distance;
		float minattack,maxattack;
		float attacktimer;
		pos = Commands->Get_Position(obj);
		pos2 = Commands->Get_Position(seen);
		distance = Commands->Get_Distance(pos2,pos);
		minattack = Get_Float_Parameter("MinAttackDistance");
		if (distance >= minattack)
		{
			maxattack = Get_Float_Parameter("MaxAttackDistance");
			params.Set_Basic(this,100,2,-1);
			params.Set_Attack_Hold(seen,maxattack,0,true,false);
			Commands->Action_Attack(obj,params);
			attacktimer = Get_Float_Parameter("AttackTimer");
			Commands->Start_Timer(obj,this,attacktimer,2);
		}
	}
}

void JFW_Base_Defence_No_VTOL::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_Base_Defence_No_VTOL::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		switch (__min(int(f),2))
		{
		case 0:
			object = Commands->Find_Object(id1);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		case 1:
			object = Commands->Find_Object(id2);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		default:
			object = Commands->Find_Object(id3);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
		}
	Commands->Start_Timer(obj,this,10,1);
	break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_Base_Defence_No_VTOL::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
}

void JFW_Base_Defence_VTOL_Only::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	objtype = Commands->Get_Player_Type(obj);
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_Base_Defence_VTOL_Only::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (Is_VTOLVehicle(seen))
	{
		ActionParamsStruct params;
		Vector3 pos,pos2;
		float distance;
		float minattack,maxattack;
		float attacktimer;
		pos = Commands->Get_Position(obj);
		pos2 = Commands->Get_Position(seen);
		distance = Commands->Get_Distance(pos2,pos);
		minattack = Get_Float_Parameter("MinAttackDistance");
		if (distance >= minattack)
		{
			maxattack = Get_Float_Parameter("MaxAttackDistance");
			params.Set_Basic(this,100,2,-1);
			params.Set_Attack_Hold(seen,maxattack,0,true,false);
			Commands->Action_Attack(obj,params);
			attacktimer = Get_Float_Parameter("AttackTimer");
			Commands->Start_Timer(obj,this,attacktimer,2);
		}
	}
}

void JFW_Base_Defence_VTOL_Only::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_Base_Defence_VTOL_Only::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		switch (__min(int(f),2))
		{
		case 0:
			object = Commands->Find_Object(id1);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		case 1:
			object = Commands->Find_Object(id2);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		default:
			object = Commands->Find_Object(id3);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
		}
	Commands->Start_Timer(obj,this,10,1);
	break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_Base_Defence_VTOL_Only::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
}

void JFW_Base_Defence_No_VTOL_Secondary::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	primary = true;
	objtype = Commands->Get_Player_Type(obj);
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_Base_Defence_No_VTOL_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (!Is_VTOLVehicle(seen))
	{
		ActionParamsStruct params;
		Vector3 pos,pos2;
		float distance;
		float minattack,maxattack;
		float attacktimer;
		pos = Commands->Get_Position(obj);
		pos2 = Commands->Get_Position(seen);
		distance = Commands->Get_Distance(pos2,pos);
		minattack = Get_Float_Parameter("MinAttackDistance");
		if (distance >= minattack)
		{
			maxattack = Get_Float_Parameter("MaxAttackDistance");
			params.Set_Basic(this,100,2,-1);
			params.Set_Attack_Hold(seen,maxattack,0,primary,false);
			Commands->Action_Attack(obj,params);
			attacktimer = Get_Float_Parameter("AttackTimer");
			Commands->Start_Timer(obj,this,attacktimer,2);
			if (primary)
			{
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_No_VTOL_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_Base_Defence_No_VTOL_Secondary::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		switch (__min(int(f),2))
		{
		case 0:
			object = Commands->Find_Object(id1);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		case 1:
			object = Commands->Find_Object(id2);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		default:
			object = Commands->Find_Object(id3);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
		}
	Commands->Start_Timer(obj,this,10,1);
	break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_Base_Defence_No_VTOL_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
	Auto_Save_Variable(5,1,&primary);
}

void JFW_Base_Defence_VTOL_Only_Secondary::Created(GameObject *obj)
{
	Vector3 pos,pos1,pos2,pos3;
	GameObject *object;
	primary = true;
	objtype = Commands->Get_Player_Type(obj);
	Commands->Enable_Hibernation(obj,false);
	Commands->Innate_Enable(obj);
	Commands->Enable_Enemy_Seen(obj,true);
	pos = Commands->Get_Position(obj);
	pos1.X = pos.X - 10;
	pos1.Y = pos.Y - 10;
	pos1.Z = pos.Z + 2;
	pos2.X = pos.X + 10;
	pos2.Y = pos.Y;
	pos2.Z = pos.Z + 2;
	pos3.X = pos.X + 10;
	pos3.Y = pos.Y - 10;
	pos3.Z = pos.Z + 2;
	object = Commands->Create_Object("Invisible_Object",pos1);
	if (object)
		id1 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos2);
	if (object)
		id2 = Commands->Get_ID(object);
	object = Commands->Create_Object("Invisible_Object",pos3);
	if (object)
		id3 = Commands->Get_ID(object);
	Commands->Start_Timer(obj,this,10,1);
}

void JFW_Base_Defence_VTOL_Only_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (Is_VTOLVehicle(seen))
	{
		ActionParamsStruct params;
		Vector3 pos,pos2;
		float distance;
		float minattack,maxattack;
		float attacktimer;
		pos = Commands->Get_Position(obj);
		pos2 = Commands->Get_Position(seen);
		distance = Commands->Get_Distance(pos2,pos);
		minattack = Get_Float_Parameter("MinAttackDistance");
		if (distance >= minattack)
		{
			maxattack = Get_Float_Parameter("MaxAttackDistance");
			params.Set_Basic(this,100,2,-1);
			params.Set_Attack_Hold(seen,maxattack,0,primary,false);
			Commands->Action_Attack(obj,params);
			attacktimer = Get_Float_Parameter("AttackTimer");
			Commands->Start_Timer(obj,this,attacktimer,2);
			if (primary)
			{
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_VTOL_Only_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	if (action == 2)
	{
		Commands->Action_Reset(obj,100);
	}
}

void JFW_Base_Defence_VTOL_Only_Secondary::Timer_Expired(GameObject *obj,int number)
{
	ActionParamsStruct var;
	GameObject *object;
	float f;
	switch (number)
	{
	case 1:
		f = Commands->Get_Random(0.0f,2.9999f);
		switch (__min(int(f),2))
		{
		case 0:
			object = Commands->Find_Object(id1);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		case 1:
			object = Commands->Find_Object(id2);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
			break;
		default:
			object = Commands->Find_Object(id3);
			if (object)
			{
				var.Set_Basic(this,70,1,-1);
				var.Set_Attack(object,0.0,0.0,true);
				Commands->Action_Attack(obj,var);
			}
		}
	Commands->Start_Timer(obj,this,10,1);
	break;
	case 2:
		Commands->Action_Reset(obj,100);
		break;
	}
}

void JFW_Base_Defence_VTOL_Only_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,4,&id1);
	Auto_Save_Variable(2,4,&id2);
	Auto_Save_Variable(3,4,&id3);
	Auto_Save_Variable(4,4,&objtype);
	Auto_Save_Variable(5,1,&primary);
}

void JFW_Base_Defence_Animated_No_VTOL::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_No_VTOL::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (!Is_VTOLVehicle(seen))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,true,false);
			Commands->Action_Attack(obj,params);
		}
	}
}

void JFW_Base_Defence_Animated_No_VTOL::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_No_VTOL::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_No_VTOL::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&attack);
}

void JFW_Base_Defence_Animated_VTOL_Only::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_VTOL_Only::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (Is_VTOLVehicle(seen))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,true,false);
			Commands->Action_Attack(obj,params);
		}
	}
}

void JFW_Base_Defence_Animated_VTOL_Only::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_VTOL_Only::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_VTOL_Only::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&attack);
}

void JFW_Base_Defence_Animated_No_VTOL_Secondary::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	primary = true;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_No_VTOL_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (!Is_VTOLVehicle(seen))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,primary,false);
			Commands->Action_Attack(obj,params);
			if (primary)
			{
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_Animated_No_VTOL_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_No_VTOL_Secondary::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_No_VTOL_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&primary);
	Auto_Save_Variable(3,1,&attack);
}

void JFW_Base_Defence_Animated_VTOL_Only_Secondary::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	primary = true;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_VTOL_Only_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (Is_VTOLVehicle(seen))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,primary,false);
			Commands->Action_Attack(obj,params);
			if (primary)
			{	
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_Animated_VTOL_Only_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_VTOL_Only_Secondary::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_VTOL_Only_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&primary);
	Auto_Save_Variable(3,1,&attack);
}

void JFW_Base_Defence_Animated_Sound::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Sound::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (!popup)
	{
		popup = true;
		Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
		Commands->Create_Sound(Get_Parameter("Sound"),Commands->Get_Position(seen),obj);
		Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
	}
	else if (attack)
	{
		ActionParamsStruct params;
		params.Set_Basic(this,100,1,-1);
		params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,true,false);
		Commands->Action_Attack(obj,params);
	}
}

void JFW_Base_Defence_Animated_Sound::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Sound::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Sound::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&attack);
}

void JFW_Base_Defence_Animated_Sound_No_Aircraft::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Sound_No_Aircraft::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Create_Sound(Get_Parameter("Sound"),Commands->Get_Position(seen),obj);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,true,false);
			Commands->Action_Attack(obj,params);
		}
	}
}

void JFW_Base_Defence_Animated_Sound_No_Aircraft::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Sound_No_Aircraft::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Sound_No_Aircraft::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&attack);
}

void JFW_Base_Defence_Animated_Sound_Aircraft_Only::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Sound_Aircraft_Only::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Create_Sound(Get_Parameter("Sound"),Commands->Get_Position(seen),obj);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,true,false);
			Commands->Action_Attack(obj,params);
		}
	}
}

void JFW_Base_Defence_Animated_Sound_Aircraft_Only::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Sound_Aircraft_Only::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Sound_Aircraft_Only::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&attack);
}

void JFW_Base_Defence_Animated_Sound_Secondary::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	primary = true;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Sound_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (!popup)
	{
		popup = true;
		Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
		Commands->Create_Sound(Get_Parameter("Sound"),Commands->Get_Position(seen),obj);
		Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
	}
	else if (attack)
	{
		ActionParamsStruct params;
		params.Set_Basic(this,100,1,-1);
		params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,primary,false);
		Commands->Action_Attack(obj,params);
		if (primary)
		{
			primary = false;
		}
		else
		{
			primary = true;
		}
	}
}

void JFW_Base_Defence_Animated_Sound_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Sound_Secondary::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Sound_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&primary);
	Auto_Save_Variable(3,1,&attack);
}

void JFW_Base_Defence_Animated_Sound_No_Aircraft_Secondary::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	primary = true;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Sound_No_Aircraft_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) && (stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Create_Sound(Get_Parameter("Sound"),Commands->Get_Position(seen),obj);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,primary,false);
			Commands->Action_Attack(obj,params);
			if (primary)
			{
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_Animated_Sound_No_Aircraft_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Sound_No_Aircraft_Secondary::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Sound_No_Aircraft_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&primary);
	Auto_Save_Variable(3,1,&attack);
}

void JFW_Base_Defence_Animated_Sound_Aircraft_Only_Secondary::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	primary = true;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Sound_Aircraft_Only_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if ((!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset1"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset2"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset3"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset4"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset5"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset6"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset7"))) || (!stricmp(Commands->Get_Preset_Name(seen),Get_Parameter("Preset8"))))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Create_Sound(Get_Parameter("Sound"),Commands->Get_Position(seen),obj);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,primary,false);
			Commands->Action_Attack(obj,params);
			if (primary)
			{	
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_Animated_Sound_Aircraft_Only_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Sound_Aircraft_Only_Secondary::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Sound_Aircraft_Only_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&primary);
	Auto_Save_Variable(3,1,&attack);
}

void JFW_Base_Defence_Animated_Sound_No_VTOL::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Sound_No_VTOL::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (!Is_VTOLVehicle(seen))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Create_Sound(Get_Parameter("Sound"),Commands->Get_Position(seen),obj);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,true,false);
			Commands->Action_Attack(obj,params);
		}
	}
}

void JFW_Base_Defence_Animated_Sound_No_VTOL::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Sound_No_VTOL::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Sound_No_VTOL::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&attack);
}

void JFW_Base_Defence_Animated_Sound_VTOL_Only::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Sound_VTOL_Only::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (Is_VTOLVehicle(seen))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Create_Sound(Get_Parameter("Sound"),Commands->Get_Position(seen),obj);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,true,false);
			Commands->Action_Attack(obj,params);
		}
	}
}

void JFW_Base_Defence_Animated_Sound_VTOL_Only::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Sound_VTOL_Only::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Sound_VTOL_Only::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&attack);
}

void JFW_Base_Defence_Animated_Sound_No_VTOL_Secondary::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	primary = true;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Sound_No_VTOL_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (!Is_VTOLVehicle(seen))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Create_Sound(Get_Parameter("Sound"),Commands->Get_Position(seen),obj);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,primary,false);
			Commands->Action_Attack(obj,params);
			if (primary)
			{
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_Animated_Sound_No_VTOL_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Sound_No_VTOL_Secondary::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Sound_No_VTOL_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&primary);
	Auto_Save_Variable(3,1,&attack);
}

void JFW_Base_Defence_Animated_Sound_VTOL_Only_Secondary::Created(GameObject *obj)
{
	popup = false;
	attack = false;
	primary = true;
	Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,0,false);
	Commands->Enable_Enemy_Seen(obj,true);
}

void JFW_Base_Defence_Animated_Sound_VTOL_Only_Secondary::Enemy_Seen(GameObject *obj,GameObject *seen)
{
	GameObject *o = Get_Vehicle(seen);
	if (o)
	{
		seen = o;
	}
	if (Is_VTOLVehicle(seen))
	{
		if (!popup)
		{
			popup = true;
			Commands->Start_Timer(obj,this,Get_Float_Parameter("PopupTime"),2);
			Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,0,Get_Float_Parameter("LastFrame"),false);
			Commands->Create_Sound(Get_Parameter("Sound"),Commands->Get_Position(seen),obj);
			Commands->Start_Timer(obj,this,Get_Float_Parameter("AttackTimer"),1);
		}
		else if (attack)
		{
			ActionParamsStruct params;
			params.Set_Basic(this,100,1,-1);
			params.Set_Attack_Hold(seen,Get_Float_Parameter("MaxAttackDistance"),0,primary,false);
			Commands->Action_Attack(obj,params);
			if (primary)
			{	
				primary = false;
			}
			else
			{
				primary = true;
			}
		}
	}
}

void JFW_Base_Defence_Animated_Sound_VTOL_Only_Secondary::Action_Complete(GameObject *obj,int action,ActionCompleteReason reason)
{
	Commands->Action_Reset(obj,100);
}

void JFW_Base_Defence_Animated_Sound_VTOL_Only_Secondary::Timer_Expired(GameObject *obj,int number)
{
	if (number == 1)
	{
		popup = false;
		attack = false;
		Commands->Set_Animation(obj,Get_Parameter("Animation"),false,0,Get_Float_Parameter("LastFrame"),0,false);
		Commands->Action_Reset(obj,100);
	}
	if (number == 2)
	{
		attack = true;
	}
}

void JFW_Base_Defence_Animated_Sound_VTOL_Only_Secondary::Register_Auto_Save_Variables()
{
	Auto_Save_Variable(1,1,&popup);
	Auto_Save_Variable(2,1,&primary);
	Auto_Save_Variable(3,1,&attack);
}

ScriptRegistrant<JFW_User_Controllable_Base_Defence> JFW_User_Controllable_Base_Defence_Registrant("JFW_User_Controllable_Base_Defence","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float");
ScriptRegistrant<JFW_Base_Defence> JFW_Base_Defence_Registrant("JFW_Base_Defence","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float");
ScriptRegistrant<JFW_Base_Defence_No_Aircraft> JFW_Base_Defence_No_Aircraft_Registrant("JFW_Base_Defence_No_Aircraft","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string");
ScriptRegistrant<JFW_Base_Defence_Aircraft_Only> JFW_Base_Defence_Aircraft_Only_Registrant("JFW_Base_Defence_Aircraft_Only","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string");
ScriptRegistrant<JFW_Base_Defence_Secondary> JFW_Base_Defence_Secondary_Registrant("JFW_Base_Defence_Secondary","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float");
ScriptRegistrant<JFW_Base_Defence_No_Aircraft_Secondary> JFW_Base_Defence_No_Aircraft_Secondary_Registrant("JFW_Base_Defence_No_Aircraft_Secondary","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string");
ScriptRegistrant<JFW_Base_Defence_Aircraft_Only_Secondary> JFW_Base_Defence_Aircraft_Only_Secondary_Registrant("JFW_Base_Defence_Aircraft_Only_Secondary","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string");
ScriptRegistrant<JFW_Base_Defence_Animated> JFW_Base_Defence_Animated_Registrant("JFW_Base_Defence_Animated","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float");
ScriptRegistrant<JFW_Base_Defence_Animated_No_Aircraft> JFW_Base_Defence_Animated_No_Aircraft_Registrant("JFW_Base_Defence_Animated_No_Aircraft","MaxAttackDistance:float,AttackTimer:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string,Animation:string,LastFrame:float,PopupTime:float");
ScriptRegistrant<JFW_Base_Defence_Animated_Aircraft_Only> JFW_Base_Defence_Animated_Aircraft_Only_Registrant("JFW_Base_Defence_Animated_Aircraft_Only","MaxAttackDistance:float,AttackTimer:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string,Animation:string,LastFrame:float,PopupTime:float");
ScriptRegistrant<JFW_Base_Defence_Animated_Secondary> JFW_Base_Defence_Animated_Secondary_Registrant("JFW_Base_Defence_Animated_Secondary","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float");
ScriptRegistrant<JFW_Base_Defence_Animated_No_Aircraft_Secondary> JFW_Base_Defence_Animated_No_Aircraft_Secondary_Registrant("JFW_Base_Defence_Animated_No_Aircraft_Secondary","MaxAttackDistance:float,AttackTimer:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string,Animation:string,LastFrame:float,PopupTime:float");
ScriptRegistrant<JFW_Base_Defence_Animated_Aircraft_Only_Secondary> JFW_Base_Defence_Animated_Aircraft_Only_Secondary_Registrant("JFW_Base_Defence_Animated_Aircraft_Only_Secondary","MaxAttackDistance:float,AttackTimer:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string,Animation:string,LastFrame:float,PopupTime:float");
ScriptRegistrant<JFW_Base_Defence_No_VTOL> JFW_Base_Defence_No_VTOL_Registrant("JFW_Base_Defence_No_VTOL","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float");
ScriptRegistrant<JFW_Base_Defence_VTOL_Only> JFW_Base_Defence_VTOL_Only_Registrant("JFW_Base_Defence_VTOL_Only","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float");
ScriptRegistrant<JFW_Base_Defence_No_VTOL_Secondary> JFW_Base_Defence_No_VTOL_Secondary_Registrant("JFW_Base_Defence_No_VTOL_Secondary","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float");
ScriptRegistrant<JFW_Base_Defence_VTOL_Only_Secondary> JFW_Base_Defence_VTOL_Only_Secondary_Registrant("JFW_Base_Defence_VTOL_Only_Secondary","MinAttackDistance=0.0:float,MaxAttackDistance=300.0:float,AttackTimer=10.0:float");
ScriptRegistrant<JFW_Base_Defence_Animated_No_VTOL> JFW_Base_Defence_Animated_No_VTOL_Registrant("JFW_Base_Defence_Animated_No_VTOL","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float");
ScriptRegistrant<JFW_Base_Defence_Animated_VTOL_Only> JFW_Base_Defence_Animated_VTOL_Only_Registrant("JFW_Base_Defence_Animated_VTOL_Only","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float");
ScriptRegistrant<JFW_Base_Defence_Animated_No_VTOL_Secondary> JFW_Base_Defence_Animated_No_VTOL_Secondary_Registrant("JFW_Base_Defence_Animated_No_VTOL_Secondary","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float");
ScriptRegistrant<JFW_Base_Defence_Animated_VTOL_Only_Secondary> JFW_Base_Defence_Animated_VTOL_Only_Secondary_Registrant("JFW_Base_Defence_Animated_VTOL_Only_Secondary","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float");
ScriptRegistrant<JFW_Base_Defence_Animated_Sound> JFW_Base_Defence_Animated_Sound_Registrant("JFW_Base_Defence_Animated_Sound","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float,Sound:string");
ScriptRegistrant<JFW_Base_Defence_Animated_Sound_No_Aircraft> JFW_Base_Defence_Animated_Sound_No_Aircraft_Registrant("JFW_Base_Defence_Animated_Sound_No_Aircraft","MaxAttackDistance:float,AttackTimer:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string,Animation:string,LastFrame:float,PopupTime:float,Sound:string");
ScriptRegistrant<JFW_Base_Defence_Animated_Sound_Aircraft_Only> JFW_Base_Defence_Animated_Sound_Aircraft_Only_Registrant("JFW_Base_Defence_Animated_Sound_Aircraft_Only","MaxAttackDistance:float,AttackTimer:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string,Animation:string,LastFrame:float,PopupTime:float,Sound:string");
ScriptRegistrant<JFW_Base_Defence_Animated_Sound_Secondary> JFW_Base_Defence_Animated_Sound_Secondary_Registrant("JFW_Base_Defence_Animated_Sound_Secondary","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float,Sound:string");
ScriptRegistrant<JFW_Base_Defence_Animated_Sound_No_Aircraft_Secondary> JFW_Base_Defence_Animated_Sound_No_Aircraft_Secondary_Registrant("JFW_Base_Defence_Animated_Sound_No_Aircraft_Secondary","MaxAttackDistance:float,AttackTimer:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string,Animation:string,LastFrame:float,PopupTime:float,Sound:string");
ScriptRegistrant<JFW_Base_Defence_Animated_Sound_Aircraft_Only_Secondary> JFW_Base_Defence_Animated_Sound_Aircraft_Only_Secondary_Registrant("JFW_Base_Defence_Animated_Sound_Aircraft_Only_Secondary","MaxAttackDistance:float,AttackTimer:float,Preset1:string,Preset2:string,Preset3:string,Preset4:string,Preset5:string,Preset6:string,Preset7:string,Preset8:string,Animation:string,LastFrame:float,PopupTime:float,Sound:string");
ScriptRegistrant<JFW_Base_Defence_Animated_Sound_No_VTOL> JFW_Base_Defence_Animated_Sound_No_VTOL_Registrant("JFW_Base_Defence_Animated_Sound_No_VTOL","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float,Sound:string");
ScriptRegistrant<JFW_Base_Defence_Animated_Sound_VTOL_Only> JFW_Base_Defence_Animated_Sound_VTOL_Only_Registrant("JFW_Base_Defence_Animated_Sound_VTOL_Only","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float,Sound:string");
ScriptRegistrant<JFW_Base_Defence_Animated_Sound_No_VTOL_Secondary> JFW_Base_Defence_Animated_Sound_No_VTOL_Secondary_Registrant("JFW_Base_Defence_Animated_Sound_No_VTOL_Secondary","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float,Sound:string");
ScriptRegistrant<JFW_Base_Defence_Animated_Sound_VTOL_Only_Secondary> JFW_Base_Defence_Animated_Sound_VTOL_Only_Secondary_Registrant("JFW_Base_Defence_Animated_Sound_VTOL_Only_Secondary","MaxAttackDistance:float,AttackTimer:float,Animation:string,LastFrame:float,PopupTime:float,Sound:string");
